{% extends "base.html" %}

{% block title %}{{ title }} - ERROR{% endblock %}

{% block content %}
    <h1>{{ title }} - ERROR</h1>
    <h2>{{ message }}</h2>
    <p><a href="{{ url_for('website.index') }}">Back</a></p>
{%- endblock %}
