""" IBKR Report Parser

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see <https://www.gnu.org/licenses/>.
"""
from ibkr_report import create_app, definitions


def main() -> None:
    """This is used when running the module locally."""
    app = create_app()
    app.run(host="127.0.0.1", port=8080, debug=definitions.DEBUG)


if __name__ == "__main__":
    main()
