body {
  font-family: Georgia, Times, serif;
  color: #333333;
  background-color: #f0f0f0;
  text-align: center;
}
h1 {
  color: #333333;
  font-family: 'Bitter', serif;
  font-size: 50px;
  font-weight: normal;
  line-height: 54px;
  margin: 0 0 54px;
}
h2, h3 {
  color: #333333;
  font-family: 'Bitter', serif;
}
p {
  color: #333333;
  font-family: Georgia, serif;
  font-size: 18px;
  line-height: 28px;
  margin: 0 auto 28px;
  max-width: 80%;
}
a {
  color: #c73036;
  font-family: Georgia, serif;
  text-decoration: underline;
}
a:hover {
  color: #333333;
  text-decoration: underline;
}
form {
  display: inline-block;
}
img {
  height: auto;
  max-width: 100%;
}
input[type="file"] {
  display: none;
}
.file-button {
  appearance: none;
  background-color: #FAFBFC;
  border: 1px solid rgba(27, 31, 35, 0.15);
  border-radius: 6px;
  box-shadow: rgba(27, 31, 35, 0.04) 0 1px 0, rgba(255, 255, 255, 0.25) 0 1px 0 inset;
  box-sizing: border-box;
  color: #24292E;
  cursor: pointer;
  display: inline-block;
  font-family: system-ui, "Segoe UI", Helvetica, Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji";
  font-size: 14px;
  font-weight: 500;
  line-height: 20px;
  list-style: none;
  padding: 6px 16px;
  position: relative;
  transition: background-color 0.2s cubic-bezier(0.3, 0, 0.5, 1);
  user-select: none;
  touch-action: manipulation;
  vertical-align: middle;
  white-space: nowrap;
  word-wrap: break-word;
}
.file-button:hover {
  background-color: #F3F4F6;
  text-decoration: none;
  transition-duration: 0.1s;
}
.file-button:disabled {
  background-color: #FAFBFC;
  border-color: rgba(27, 31, 35, 0.15);
  color: #959DA5;
  cursor: default;
}
.file-button:active {
  background-color: #EDEFF2;
  box-shadow: rgba(225, 228, 232, 0.2) 0 1px 0 inset;
  transition: none 0s;
}
.file-button:focus {
  outline: 1px transparent;
}
.file-button:before {
  display: none;
}
.styled-table {
  border-collapse: collapse;
  margin: 25px auto;
  font-size: 0.9em;
  font-family: sans-serif;
  min-width: 400px;
  box-shadow: 0 0 20px rgba(0, 0, 0, 0.15);
  text-align: left;
}
.styled-table thead tr {
  background-color: #007998;
  color: #ffffff;
}
.styled-table th, .styled-table td {
  padding: 12px 15px;
}
.styled-table tbody tr {
  border-bottom: 1px solid #dddddd;
}
.styled-table tbody tr:nth-of-type(even) {
  background-color: #f3f3f3;
}
.styled-table tbody tr:last-of-type {
  border-bottom: 2px solid #007998;
}
